import 'package:eyflutter_core/provider/cus_change_notifier.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

typedef Widget ConsumerElementBuilder<M extends ElementModel>(BuildContext context, M model);
typedef Widget ConsumerItemBuilder<M extends ElementModel, I>(BuildContext context, M model, int position, I item);

class ElementModel<P> extends CusChangeNotifier {
  late P _parent;
  dynamic _source;

  P get parent => _parent;

  ElementModel(P parent) {
    this._parent = parent;
  }

  //设置数据源
  void setSource<D>(D? data) {
    _source = data;
  }
}

class _StateWidget {
  _StateWidget._();

  static Widget subscribeNotifierWidget<M extends ElementModel>(M model, ConsumerElementBuilder<M> childBuilder) {
    return ChangeNotifierProvider<M>(
      create: (_) => model,
      child: Consumer<M>(
        builder: (context, m, _) {
          return childBuilder(context, m);
        },
      ),
    );
  }

// static Widget subscribeItemWidget<M extends ElementModel, I>(
//     M model, int position, I item, ConsumerItemBuilder<M, I> childBuilder) {
//   return ChangeNotifierProvider<M>(
//     create: (_) => model,
//     child: Consumer<M>(
//       builder: (context, m, _) {
//         return childBuilder(context, m, position, item);
//       },
//     ),
//   );
// }
}

abstract class ElementBuilder<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  D? _source;

  D? get source => _getSource();

  D? _getSource() {
    var mSource = _model._source;
    if (mSource is D) {
      _source = mSource;
    }
    return _source;
  }

  void setSource(D? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  /// 构建初始化
  void onBuildInit() {}

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

abstract class ElementsBuilder<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  List<I>? _source;

  List<I>? get source => _source;

  void setSource(List<I>? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  Widget itemBuilder(BuildContext context, int position, I item);

  void itemProcessing(I item);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

/// 元素状态
class ElementState<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  ElementState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidget({required BuildContext context, D? source, required ElementBuilder<M, D> builder}) {
    if (source != null) {
      builder.setSource(source);
    }
    builder.setModel(_model);
    builder.onBuildInit();
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}

/// 元素状态
class ElementsState<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  ElementsState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件集转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidgets({required BuildContext context, List<I>? source, required ElementsBuilder<M, I> builder}) {
    builder.setSource(source);
    builder.setModel(this._model);
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}
